@@ -205,12 +205,17 @@ class JobFeatureTests:
         assert job.status == "finished"
         workers = job.workers
         await job.remove()
+        captured = capsys.readouterr()
+        sys.stdout.write(captured.out)
+        sys.stderr.write(captured.err)
+        assert "Epoch 1/2" in captured.out
+        assert "Epoch 2/2" in captured.out
 
         new_model = await trainml.models.get(
             workers[0].get("output_model_uuid")
         )
         assert new_model.id
-        await new_model.attach()
+        await new_model.wait_for("ready")
         await new_model.refresh()
         assert new_model.size > model.size + 1000000
         assert (
@@ -218,10 +223,3 @@ class JobFeatureTests:
             == "Job - CLI Automated Training With trainML Model Output"
         )
         await new_model.remove()
-
-        captured = capsys.readouterr()
-        sys.stdout.write(captured.out)
-        sys.stderr.write(captured.err)
-        assert "Epoch 1/2" in captured.out
-        assert "Epoch 2/2" in captured.out
-        assert "Finish copy worker contents" in captured.out
