@@ -3,7 +3,9 @@ from pytest import fixture, mark
 from unittest.mock import Mock, AsyncMock, patch, create_autospec
 
 from trainml.trainml import TrainML
+from trainml.auth import Auth
 from trainml.datasets import Dataset, Datasets
+from trainml.models import Model, Models
 from trainml.gpu_types import GpuType, GpuTypes
 from trainml.environments import Environment, Environments
 from trainml.jobs import Job, Jobs
@@ -15,7 +17,7 @@ pytestmark = mark.unit
 @fixture(scope="session")
 def mock_my_datasets():
     trainml = Mock()
-    return [
+    yield [
         Dataset(
             trainml,
             dataset_uuid="1",
@@ -82,7 +84,7 @@ def mock_my_datasets():
 @fixture(scope="session")
 def mock_public_datasets():
     trainml = Mock()
-    return [
+    yield [
         Dataset(
             trainml,
             dataset_uuid="11",
@@ -128,10 +130,77 @@ def mock_public_datasets():
     ]
 
 
+@fixture(scope="session")
+def mock_models():
+    trainml = Mock()
+    yield [
+        Model(
+            trainml,
+            model_uuid="1",
+            customer_uuid="cus-id-1",
+            name="first one",
+            status="ready",
+            provider="trainml",
+            size=10000000,
+            createdAt="2020-12-31T23:59:59.000Z",
+        ),
+        Model(
+            trainml,
+            model_uuid="2",
+            customer_uuid="cus-id-1",
+            name="second one",
+            status="ready",
+            provider="trainml",
+            size=10000000,
+            createdAt="2021-01-01T00:00:01.000Z",
+        ),
+        Model(
+            trainml,
+            model_uuid="3",
+            customer_uuid="cus-id-1",
+            name="first one",
+            status="ready",
+            provider="gcp",
+            size=10000000,
+            createdAt="2021-01-01T00:00:01.000Z",
+        ),
+        Model(
+            trainml,
+            model_uuid="4",
+            customer_uuid="cus-id-1",
+            name="other one",
+            status="ready",
+            provider="gcp",
+            size=10000000,
+            createdAt="2020-12-31T23:59:59.000Z",
+        ),
+        Model(
+            trainml,
+            model_uuid="5",
+            customer_uuid="cus-id-1",
+            name="not ready",
+            status="new",
+            provider="trainml",
+            size=10000000,
+            createdAt="2021-01-01T00:00:01.000Z",
+        ),
+        Model(
+            trainml,
+            model_uuid="6",
+            customer_uuid="cus-id-1",
+            name="failed",
+            status="failed",
+            provider="trainml",
+            size=10000000,
+            createdAt="2021-01-01T00:00:01.000Z",
+        ),
+    ]
+
+
 @fixture(scope="session")
 def mock_gpu_types():
     trainml = Mock()
-    return [
+    yield [
         GpuType(
             trainml,
             **{
@@ -198,7 +267,7 @@ def mock_gpu_types():
 @fixture(scope="function")
 def mock_environments():
     trainml = Mock()
-    return [
+    yield [
         Environment(
             trainml,
             **{
@@ -291,7 +360,7 @@ def mock_environments():
 @fixture(scope="session")
 def mock_jobs():
     trainml = Mock()
-    return [
+    yield [
         Job(
             trainml,
             **{
@@ -446,19 +515,22 @@ def mock_jobs():
 def mock_trainml(
     mock_my_datasets,
     mock_public_datasets,
+    mock_models,
     mock_gpu_types,
     mock_environments,
     mock_jobs,
 ):
     trainml = create_autospec(TrainML)
     trainml.datasets = create_autospec(Datasets)
+    trainml.models = create_autospec(Models)
     trainml.gpu_types = create_autospec(GpuTypes)
     trainml.environments = create_autospec(Environments)
     trainml.jobs = create_autospec(Jobs)
     trainml.connections = create_autospec(Connections)
     trainml.datasets.list = AsyncMock(return_value=mock_my_datasets)
     trainml.datasets.list_public = AsyncMock(return_value=mock_public_datasets)
+    trainml.models.list = AsyncMock(return_value=mock_models)
     trainml.gpu_types.list = AsyncMock(return_value=mock_gpu_types)
     trainml.environments.list = AsyncMock(return_value=mock_environments)
     trainml.jobs.list = AsyncMock(return_value=mock_jobs)
-    return trainml
\ No newline at end of file
+    yield trainml
